@@ -10,23 +10,34 @@ use core::convert::Infallible;
 /// This trait is also implemented by `bool` and `u8`. While `u8` is
 /// strictly speaking not an actual enum, there are good reasons to consider
 /// it like one, as array of `u8` keys is a relatively common pattern.
-pub trait Enum<V>: Sized {
-    /// Representation of an enum map for type `V`.
-    type Array: Array<V>;
+pub trait Enum: Sized {
+    /// Length of the enum.
+    const LENGTH: usize;
+
     /// Takes an usize, and returns an element matching `into_usize` function.
     fn from_usize(value: usize) -> Self;
     /// Returns an unique identifier for a value within range of `0..Array::LENGTH`.
     fn into_usize(self) -> usize;
 }
 
+/// Trait associating
+pub trait EnumArray<V>: Enum {
+    /// Representation of an enum map for type `V`.
+    type Array: Array<V>;
+}
+
+/// Array for enum-map storage.
+///
+/// This trait is inteded for primitive array types (with fixed length).
 pub trait Array<V> {
-    const LENGTH: usize;
+    /// Coerces a reference to the array into a reference to a slice.
     fn slice(&self) -> &[V];
+
+    /// Coerces a mutable reference to the array into a mutable reference to a slice.
     fn slice_mut(&mut self) -> &mut [V];
 }
 
 impl<V, const N: usize> Array<V> for [V; N] {
-    const LENGTH: usize = N;
     fn slice(&self) -> &[V] {
         self
     }
@@ -35,8 +46,9 @@ impl<V, const N: usize> Array<V> for [V; N] {
     }
 }
 
-impl<T> Enum<T> for bool {
-    type Array = [T; 2];
+impl Enum for bool {
+    const LENGTH: usize = 2;
+
     #[inline]
     fn from_usize(value: usize) -> Self {
         match value {
@@ -51,8 +63,13 @@ impl<T> Enum<T> for bool {
     }
 }
 
-impl<T> Enum<T> for u8 {
-    type Array = [T; 256];
+impl<T> EnumArray<T> for bool {
+    type Array = [T; Self::LENGTH];
+}
+
+impl Enum for u8 {
+    const LENGTH: usize = 256;
+
     #[inline]
     fn from_usize(value: usize) -> Self {
         value as u8
@@ -63,8 +80,13 @@ impl<T> Enum<T> for u8 {
     }
 }
 
-impl<T> Enum<T> for Infallible {
-    type Array = [T; 0];
+impl<T> EnumArray<T> for u8 {
+    type Array = [T; Self::LENGTH];
+}
+
+impl Enum for Infallible {
+    const LENGTH: usize = 0;
+
     #[inline]
     fn from_usize(_: usize) -> Self {
         unreachable!();
@@ -74,3 +96,7 @@ impl<T> Enum<T> for Infallible {
         match self {}
     }
 }
+
+impl<T> EnumArray<T> for Infallible {
+    type Array = [T; Self::LENGTH];
+}
